"""Packing Packages

Packing packages for distribution and installation.
This module provides functionality to pack conda environments and their
dependencies into a specified directory.

.. code-block:: bash

    $ python -m packing_packages pack --help
    $ python -m packing_packages install --help

    or

    $ packing-packages pack --help
    $ packing-packages install --help

"""

__version__ = "0.4.1"
__license__ = "MIT"
__author__ = "yu9824"
__copyright__ = "Copyright © 2025 yu9824"

# __all__ = ()
